@@ -40,36 +40,62 @@ class InitCommand:
             spthy_files: List of spthy file paths
             output_path: Optional output path for the generated config
         """
-        self.console.print(
-            Panel.fit(
-                "[bold blue]Batch Tamarin Configuration Generator[/bold blue]",
-                style="blue",
+        try:
+            self.console.print(
+                Panel.fit(
+                    "[bold blue]Batch Tamarin Configuration Generator[/bold blue]",
+                    style="blue",
+                )
             )
-        )
 
-        # Validate spthy files exist
-        validated_files = self._validate_spthy_files(spthy_files)
-        if not validated_files:
-            self.console.print("[red]No valid spthy files provided![/red]")
+            # Validate spthy files exist
+            validated_files = self._validate_spthy_files(spthy_files)
+            if not validated_files:
+                self.console.print("[red]No valid spthy files provided![/red]")
+                return
+
+            # Collect configuration interactively with fallbacks
+            config = self._collect_global_config_with_fallback()
+            # store for use in task-specific resource defaults
+            self._global_config = config
+            tamarin_versions = self._collect_tamarin_versions_with_fallback()
+            tasks, failed_files = self._collect_tasks(
+                validated_files, list(tamarin_versions.keys())
+            )
+
+            # Display summary of failed files
+            if failed_files:
+                self._display_failed_files_summary(failed_files)
+
+            # Check if we have any valid tasks
+            if not tasks:
+                self.console.print(
+                    "[red]No valid tasks were created. Configuration file not generated.[/red]"
+                )
+                return
+
+            # Build the recipe
+            recipe = TamarinRecipe(
+                config=config, tamarin_versions=tamarin_versions, tasks=tasks
+            )
+
+            # Save or output
+            if output_path:
+                self._save_config(recipe, output_path)
+            else:
+                self._save_config(recipe, "recipe.json")
+
+        except KeyboardInterrupt:
+            self.console.print("\n[yellow]Initialization cancelled by user.[/yellow]")
+            return
+        except Exception as e:
+            self.console.print(
+                f"\n[red]Unexpected error during initialization:[/red] {e}"
+            )
+            self.console.print(
+                "[yellow]Please report this issue if it persists.[/yellow]"
+            )
             return
-
-        # Collect configuration interactively
-        config = self._collect_global_config()
-        # store for use in task-specific resource defaults
-        self._global_config = config
-        tamarin_versions = self._collect_tamarin_versions()
-        tasks = self._collect_tasks(validated_files, list(tamarin_versions.keys()))
-
-        # Build the recipe
-        recipe = TamarinRecipe(
-            config=config, tamarin_versions=tamarin_versions, tasks=tasks
-        )
-
-        # Save or output
-        if output_path:
-            self._save_config(recipe, output_path)
-        else:
-            self._save_config(recipe, "recipe.json")
 
     def _validate_spthy_files(self, spthy_files: List[str]) -> List[Path]:
         """Validate that spthy files exist and are readable."""
@@ -89,6 +115,59 @@ class InitCommand:
             validated.append(path)
         return validated
 
+    def _collect_global_config_with_fallback(self) -> GlobalConfig:
+        """Collect global configuration with fallbacks for input failures."""
+        try:
+            return self._collect_global_config()
+        except (KeyboardInterrupt, EOFError):
+            self.console.print(
+                "[yellow]Using default global configuration due to input cancellation.[/yellow]"
+            )
+            # Return sensible defaults
+            system_cores = get_max_cpu_cores()
+            system_memory = get_max_memory_gb()
+            return GlobalConfig(
+                global_max_cores=system_cores,
+                global_max_memory=system_memory,
+                default_timeout=3600,
+                output_directory="result",
+            )
+        except Exception as e:
+            self.console.print(
+                f"[yellow]Error collecting global config, using defaults: {e}[/yellow]"
+            )
+            system_cores = get_max_cpu_cores()
+            system_memory = get_max_memory_gb()
+            return GlobalConfig(
+                global_max_cores=system_cores,
+                global_max_memory=system_memory,
+                default_timeout=3600,
+                output_directory="result",
+            )
+
+    def _collect_tamarin_versions_with_fallback(self) -> Dict[str, TamarinVersion]:
+        """Collect Tamarin versions with fallbacks for input failures."""
+        try:
+            return self._collect_tamarin_versions()
+        except (KeyboardInterrupt, EOFError):
+            self.console.print(
+                "[yellow]Using default Tamarin configuration due to input cancellation.[/yellow]"
+            )
+            return {
+                "default": TamarinVersion(
+                    path="tamarin-prover", version=None, test_success=None
+                )
+            }
+        except Exception as e:
+            self.console.print(
+                f"[yellow]Error collecting Tamarin versions, using defaults: {e}[/yellow]"
+            )
+            return {
+                "default": TamarinVersion(
+                    path="tamarin-prover", version=None, test_success=None
+                )
+            }
+
     def _collect_global_config(self) -> GlobalConfig:
         """Collect global configuration settings interactively."""
         self.console.print("\n[bold]Global Configuration[/bold]")
@@ -103,7 +182,9 @@ class InitCommand:
 
         # Max cores
         max_cores_input = Prompt.ask(
-            f"Maximum CPU cores limit for all (concurrent) tasks [violet bold]{escape("[int, max]")}[/violet bold]",
+            "Maximum CPU cores limit for all (concurrent) tasks [violet bold]"
+            + escape("[int, max]")
+            + "[/violet bold]",
             default="max",
             show_default=True,
         )
@@ -111,7 +192,9 @@ class InitCommand:
 
         # Max memory
         max_memory_input = Prompt.ask(
-            f"Maximum memory (GB) limit for all (concurrent) tasks [violet bold]{escape("[int, int%, max]")}[/violet bold]",
+            "Maximum memory (GB) limit for all (concurrent) tasks [violet bold]"
+            + escape("[int, int%, max]")
+            + "[/violet bold]",
             default="max",
             show_default=True,
         )
@@ -206,86 +289,212 @@ class InitCommand:
 
     def _collect_tasks(
         self, spthy_files: List[Path], tamarin_aliases: List[str]
-    ) -> Dict[str, Task]:
-        """Collect task configurations for each spthy file."""
+    ) -> tuple[Dict[str, Task], List[tuple[Path, str]]]:
+        """Collect task configurations for each spthy file.
+
+        Returns:
+            Tuple of (successful_tasks, failed_files) where failed_files contains
+            (file_path, error_message) tuples.
+        """
         self.console.print("\n[bold]Tasks Configuration[/bold]")
 
         tasks: dict[str, Task] = {}
+        failed_files: List[tuple[Path, str]] = []
 
         for spthy_file in spthy_files:
             self.console.print(
                 f"\n[bold cyan]Configuring task for:[/bold cyan] {spthy_file.name}"
             )
 
-            # Task name
-            default_task_name = spthy_file.stem
-            task_name = Prompt.ask(
-                "Task name", default=default_task_name, show_default=True
-            )
+            try:
+                # Task name with fallback
+                default_task_name = spthy_file.stem
+                try:
+                    task_name = Prompt.ask(
+                        "Task name", default=default_task_name, show_default=True
+                    )
+                except (KeyboardInterrupt, EOFError):
+                    task_name = default_task_name
+                    self.console.print(
+                        f"[yellow]Using default task name: {task_name}[/yellow]"
+                    )
 
-            # Output file prefix
-            output_prefix = Prompt.ask(
-                "Result output file prefix", default=task_name, show_default=True
-            )
+                # Output file prefix with fallback
+                try:
+                    output_prefix = Prompt.ask(
+                        "Result output file prefix",
+                        default=task_name,
+                        show_default=True,
+                    )
+                except (KeyboardInterrupt, EOFError):
+                    output_prefix = task_name
+                    self.console.print(
+                        f"[yellow]Using default output prefix: {output_prefix}[/yellow]"
+                    )
 
-            # Select Tamarin versions for this task
-            selected_versions = self._select_tamarin_versions(
-                tamarin_aliases, f"task '{task_name}'"
-            )
+                # Select Tamarin versions for this task with fallback
+                try:
+                    selected_versions = self._select_tamarin_versions(
+                        tamarin_aliases, f"task '{task_name}'"
+                    )
+                except (KeyboardInterrupt, EOFError):
+                    selected_versions = (
+                        tamarin_aliases[:1] if tamarin_aliases else ["default"]
+                    )
+                    self.console.print(
+                        f"[yellow]Using default Tamarin version: {selected_versions[0]}[/yellow]"
+                    )
 
-            # Collect task-wide parameters
-            tamarin_options = self._collect_tamarin_options(f"task '{task_name}'")
-            preprocess_flags = self._collect_preprocess_flags(f"task '{task_name}'")
-            resources = self._collect_resources(f"task '{task_name}'")
-            lemmas = self._collect_lemmas(spthy_file, preprocess_flags, tamarin_aliases)
+                # Collect task-wide parameters with fallback
+                try:
+                    preprocess_flags = self._collect_preprocess_flags(
+                        f"task '{task_name}'"
+                    )
+                except (KeyboardInterrupt, EOFError):
+                    preprocess_flags = None
+                    self.console.print(
+                        "[yellow]No preprocessor flags configured[/yellow]"
+                    )
 
-            # Create task with collected parameters
-            task = Task(
-                theory_file=str(spthy_file),
-                tamarin_versions=selected_versions,
-                output_file_prefix=output_prefix,
-                resources=resources,
-                tamarin_options=tamarin_options,
-                preprocess_flags=preprocess_flags,
-                lemmas=lemmas,
-            )
+                # Check for diff operator usage to auto-add --diff flag
+                try:
+                    parser = LemmaParser(preprocess_flags, ignore_preprocessor=True)
+                    content = parser.preprocess_includes(spthy_file)
+                    diff_detected = parser.detect_diff_operator(content)
+                    suggested_options = ["--diff"] if diff_detected else None
+                except Exception:
+                    suggested_options = None
 
-            tasks[task_name] = task
+                try:
+                    tamarin_options = self._collect_tamarin_options(
+                        f"task '{task_name}'", suggested_options
+                    )
+                except (KeyboardInterrupt, EOFError):
+                    tamarin_options = suggested_options
+                    self.console.print(
+                        "[yellow]Using default or suggested tamarin options[/yellow]"
+                    )
 
-        return tasks
+                try:
+                    resources = self._collect_resources(f"task '{task_name}'")
+                except (KeyboardInterrupt, EOFError):
+                    resources = None
+                    self.console.print(
+                        "[yellow]Using default resource allocation[/yellow]"
+                    )
 
-    def _collect_tamarin_options(self, context: str) -> Optional[List[str]]:
-        """Collect tamarin command-line options interactively."""
-        if not Confirm.ask(
-            f"Add tamarin command-line options for {context}?", default=False
-        ):
-            return None
+                try:
+                    lemmas = self._collect_lemmas(
+                        spthy_file, preprocess_flags, tamarin_aliases
+                    )
+                except (KeyboardInterrupt, EOFError):
+                    lemmas = None
+                    self.console.print(
+                        "[yellow]Using all lemmas (default behavior)[/yellow]"
+                    )
+
+                # Create task with collected parameters
+                task = Task(
+                    theory_file=str(spthy_file),
+                    tamarin_versions=selected_versions,
+                    output_file_prefix=output_prefix,
+                    resources=resources,
+                    tamarin_options=tamarin_options,
+                    preprocess_flags=preprocess_flags,
+                    lemmas=lemmas,
+                )
+
+                tasks[task_name] = task
+                self.console.print(
+                    f"[green]✓ Task '{task_name}' created successfully[/green]"
+                )
+
+            except Exception as e:
+                error_msg = str(e)
+                self.console.print(
+                    f"[red]✗ Failed to create task for {spthy_file.name}:[/red] {error_msg}"
+                )
+                failed_files.append((spthy_file, error_msg))
+                # Continue to next file instead of crashing
+                continue
+
+        return tasks, failed_files
+
+    def _display_failed_files_summary(
+        self, failed_files: List[tuple[Path, str]]
+    ) -> None:
+        """Display a summary of files that were skipped during task creation."""
+        if not failed_files:
+            return
 
         self.console.print(
-            "[dim]Enter tamarin options separated by spaces or commas[/dim]"
+            "\n[bold yellow]Files Skipped During Initialization[/bold yellow]"
+        )
+        self.console.print(
+            "[yellow]The following files could not be processed and were ignored:[/yellow]"
+        )
+
+        for file_path, error_msg in failed_files:
+            self.console.print(f"  • [red]{file_path.name}[/red]: {error_msg}")
+
+        self.console.print(
+            f"\n[yellow]Note: {len(failed_files)} file(s) were skipped, but {len(failed_files) > 0 and 'the remaining' or 'all'} valid tasks were included in the recipe.[/yellow]"
+        )
+
+    def _collect_tamarin_options(
+        self, context: str, suggested_options: Optional[List[str]] = None
+    ) -> Optional[List[str]]:
+        """Collect tamarin command-line options interactively."""
+        # Start with suggested options
+        current_options = suggested_options.copy() if suggested_options else []
+
+        # Show auto-detected options
+        if suggested_options:
+            for option in suggested_options:
+                if option == "--diff":
+                    self.console.print(
+                        f"[green]✓ Auto-added {option} flag for observational equivalence[/green]"
+                    )
+                else:
+                    self.console.print(f"[green]✓ Auto-added {option} option[/green]")
+
+        if not Confirm.ask(
+            f"Add additional tamarin command-line options for {context}?", default=False
+        ):
+            return current_options if current_options else None
+
+        self.console.print(
+            "[dim]Enter additional tamarin options separated by spaces or commas[/dim]"
         )
         self.console.print(
             "[dim]Example: '--heuristic=I --bound=5' or '--heuristic=I, --bound=5'[/dim]"
         )
 
-        options_input = Prompt.ask("Tamarin options", default="")
+        options_input = Prompt.ask("Additional tamarin options", default="")
         if not options_input.strip():
-            return None
+            return current_options if current_options else None
 
         # Split by comma or space, clean up whitespace
         if "," in options_input:
-            options = [opt.strip() for opt in options_input.split(",") if opt.strip()]
+            additional_options = [
+                opt.strip() for opt in options_input.split(",") if opt.strip()
+            ]
         else:
-            options = [opt.strip() for opt in options_input.split() if opt.strip()]
+            additional_options = [
+                opt.strip() for opt in options_input.split() if opt.strip()
+            ]
+
+        # Combine suggested and additional options
+        all_options = current_options + additional_options
 
         # Basic validation - warn if options don't start with - or --
-        for option in options:
+        for option in additional_options:
             if not option.startswith("-"):
                 self.console.print(
                     f"[yellow]Warning:[/yellow] '{option}' doesn't start with '-' or '--'"
                 )
 
-        return options if options else None
+        return all_options if all_options else None
 
     def _collect_preprocess_flags(self, context: str) -> Optional[List[str]]:
         """Collect preprocessor flags interactively."""
@@ -407,7 +616,7 @@ class InitCommand:
 
         # Parse lemmas from the file
         try:
-            parser = LemmaParser(preprocess_flags)
+            parser = LemmaParser(preprocess_flags, ignore_preprocessor=True)
             all_lemmas = parser.parse_lemmas_from_file(spthy_file)
 
             if not all_lemmas:
@@ -420,6 +629,12 @@ class InitCommand:
                 f"[green]Found {len(all_lemmas)} lemmas in {spthy_file.name}[/green]"
             )
 
+            # Check if Observational_equivalence was auto-added due to diff operator detection
+            if "Observational_equivalence" in all_lemmas:
+                self.console.print(
+                    "[green]✓ Detected diff() operator - added Observational_equivalence lemma[/green]"
+                )
+
         except LemmaParsingError as e:
             self.console.print(
                 f"[red]Error parsing lemmas from {spthy_file.name}: {e}[/red]"
