@@ -16,11 +16,12 @@ from .model.executable_task import (
     TaskResult,
     TaskStatus,
 )
-from .model.tamarin_recipe import TamarinRecipe
+from .model.tamarin_recipe import SchedulingStrategy, TamarinRecipe
 from .modules.output_manager import output_manager
 from .modules.process_manager import process_manager
 from .modules.resource_manager import ResourceManager
 from .modules.task_manager import TaskManager
+from .utils.dot_utils import cleanup_empty_trace_files
 from .utils.notifications import notification_manager
 
 
@@ -33,17 +34,22 @@ class TaskRunner:
     progress updates throughout the execution process.
     """
 
-    def __init__(self, recipe: TamarinRecipe) -> None:
+    def __init__(
+        self,
+        recipe: TamarinRecipe,
+        scheduler: SchedulingStrategy = SchedulingStrategy.FIFO,
+    ) -> None:
         """
         Initialize the TaskRunner with a recipe containing global configuration.
 
         Args:
             recipe: TamarinRecipe object containing global configuration and tasks
+            scheduler: Task scheduling strategy to use
         """
         self.recipe = recipe
 
-        # Initialize ResourceManager with recipe (will validate and potentially correct resource limits)
-        self.resource_manager = ResourceManager(recipe)
+        # Initialize ResourceManager with recipe and scheduler (will validate and potentially correct resource limits)
+        self.resource_manager = ResourceManager(recipe, scheduler)
 
         # Initialize TaskManager for execution
         self.task_manager = TaskManager()
@@ -52,17 +58,31 @@ class TaskRunner:
         output_directory = Path(recipe.config.output_directory)
         output_manager.initialize(output_directory)
 
+        # Update recipe config with actual output directory (may have changed due to timestamping)
+        actual_output_paths = output_manager.get_output_paths()
+        self.recipe.config.output_directory = str(actual_output_paths["base"])
+
         # Internal state for task management
         self._pending_tasks: List[ExecutableTask] = []
         self._running_tasks: Dict[str, asyncio.Task[TaskResult]] = {}
         self._completed_tasks: Set[str] = set()
         self._failed_tasks: Set[str] = set()
         self._task_results: Dict[str, TaskResult] = {}
+        self.completed_tasks: Set[str] = set()
+        self.failed_tasks: Set[str] = set()
+        self.task_results: Dict[str, TaskResult] = {}
 
         # Track shutdown state
         self._shutdown_requested = False
         self._force_shutdown_requested = False
         self._signal_count = 0
+        self._signal_interrupted_tasks: Set[str] = set()
+
+        # Task completion event for immediate scheduling (will be set during execution)
+        self._task_completed_event: Optional[asyncio.Event] = None
+
+        # Store scheduler mode for logging
+        self.scheduler = scheduler
 
     async def execute_all_tasks(self, tasks: List[ExecutableTask]) -> None:
         """
@@ -94,10 +114,17 @@ class TaskRunner:
         self._completed_tasks.clear()
         self._failed_tasks.clear()
         self._task_results.clear()
+        self.completed_tasks.clear()
+        self.failed_tasks.clear()
+        self.task_results.clear()
         self._shutdown_requested = False
 
+        # Initialize task completion event for this execution
+        self._task_completed_event = asyncio.Event()
+
         # Set up shutdown handler
         def signal_handler(signum: int, frame: Any) -> None:
+            _ = signum, frame  # Suppress unused parameter warnings
             self._signal_count += 1
 
             if self._signal_count == 1:
@@ -117,7 +144,7 @@ class TaskRunner:
         try:
             # Display initial status
             notification_manager.info(
-                f"[TaskRunner] Starting execution of {len(tasks)} tasks. \n"
+                f"[TaskRunner] Starting execution of {len(tasks)} tasks using {self.scheduler.value} scheduling strategy. \n"
                 f"Available resources: {self.resource_manager.get_available_cores()} cores, "
                 f"{self.resource_manager.get_available_memory()}GB memory"
             )
@@ -125,10 +152,6 @@ class TaskRunner:
             # Start main scheduling loop
             await self._execute_task_pool(tasks)
 
-            # Generate final execution summary
-            summary = self.task_manager.generate_execution_summary()
-            notification_manager.task_execution_summary(summary)
-
         except Exception as e:
             notification_manager.error(
                 f"[TaskRunner] Unexpected error during task execution: {e}"
@@ -153,9 +176,11 @@ class TaskRunner:
 
     async def _execute_task_pool(self, all_tasks: List[ExecutableTask]) -> None:
         """
-        Internal method to manage task pool execution.
+        Internal method to manage task pool execution with event-driven scheduling.
 
-        Implements the main scheduling loop:
+        Implements improved scheduling loop:
+        - Event-driven scheduling (immediate on task completion)
+        - Adaptive sleep intervals based on activity
         - Get next schedulable tasks from ResourceManager
         - Start tasks as background coroutines
         - Monitor completions and update progress
@@ -168,18 +193,32 @@ class TaskRunner:
         last_progress_update = 0
         progress_update_interval = 3.0  # Update progress every 3 seconds
 
+        # Try initial scheduling
+        await self._schedule_available_tasks()
+
         while self._should_continue_execution():
-            # Get next schedulable tasks
-            schedulable_tasks = self.resource_manager.get_next_schedulable_tasks(
-                self._pending_tasks
-            )
+            # Wait for task completion event or timeout
+            # Use shorter timeout when tasks are running (more responsive)
+            # Use longer timeout when no tasks running (less CPU usage)
+            timeout = 0.1 if self._running_tasks else 1.0
 
-            # Start new tasks as background coroutines (only if not shutting down)
-            self._start_schedulable_tasks(schedulable_tasks)
+            try:
+                if self._task_completed_event:
+                    await asyncio.wait_for(
+                        self._task_completed_event.wait(), timeout=timeout
+                    )
+                    self._task_completed_event.clear()
+                else:
+                    await asyncio.sleep(timeout)
+            except asyncio.TimeoutError:
+                pass  # Normal timeout, continue with periodic checks
 
-            # Handle completed tasks
+            # Handle completed tasks (this may free up resources)
             await self._handle_completed_tasks(all_tasks)
 
+            # Try to schedule new tasks if resources are available
+            await self._schedule_available_tasks()
+
             # Display progress update periodically
             current_time = asyncio.get_event_loop().time()
             if current_time - last_progress_update >= progress_update_interval:
@@ -190,15 +229,25 @@ class TaskRunner:
             if self._force_shutdown_requested:
                 break
 
-            # Small sleep to prevent busy loop
-            await asyncio.sleep(1)
-
         # Handle shutdown scenarios
         await self._handle_shutdown()
 
         # Final progress update
         self._display_progress_update()
 
+    async def _schedule_available_tasks(self) -> None:
+        """Schedule available tasks if resources permit."""
+        if self._shutdown_requested or self._force_shutdown_requested:
+            return
+
+        # Get next schedulable tasks
+        schedulable_tasks = self.resource_manager.get_next_schedulable_tasks(
+            self._pending_tasks
+        )
+
+        # Start new tasks as background coroutines
+        self._start_schedulable_tasks(schedulable_tasks)
+
     def _should_continue_execution(self) -> bool:
         """Check if the task pool execution should continue."""
         return (
@@ -224,7 +273,7 @@ class TaskRunner:
                 # Remove from pending tasks
                 self._pending_tasks.remove(task)
 
-                notification_manager.info(f"[TaskRunner] Started task: {task_id}")
+                notification_manager.info(f"[TaskRunner] Started subtask: {task_id}")
 
     async def _handle_completed_tasks(self, all_tasks: List[ExecutableTask]) -> None:
         """Check for and handle completed tasks."""
@@ -249,6 +298,9 @@ class TaskRunner:
                         break
 
                 if corresponding_task:
+                    # Check if task was signal interrupted and update status
+                    if task_id in self._signal_interrupted_tasks:
+                        task_result.status = TaskStatus.SIGNAL_INTERRUPTED
                     self._handle_task_completion(corresponding_task, task_result)
                 else:
                     notification_manager.error(
@@ -266,11 +318,17 @@ class TaskRunner:
             notification_manager.debug(
                 "[TaskRunner] Force shutdown requested. Killing all running tasks immediately..."
             )
+            # Mark running tasks as signal interrupted before killing
+            for task_id in self._running_tasks.keys():
+                self._signal_interrupted_tasks.add(task_id)
             await self._force_kill_all_tasks()
         elif self._shutdown_requested:
             notification_manager.debug(
                 "[TaskRunner] Graceful shutdown requested. Waiting for running tasks to complete..."
             )
+            # Mark running tasks as signal interrupted for graceful shutdown too
+            for task_id in self._running_tasks.keys():
+                self._signal_interrupted_tasks.add(task_id)
             await self._cleanup_running_tasks()
 
     async def _execute_single_task(self, task: ExecutableTask) -> TaskResult:
@@ -322,26 +380,49 @@ class TaskRunner:
         # Release resources
         self.resource_manager.release_resources(task)
 
+        # Clean up empty trace files
+        try:
+            cleanup_empty_trace_files(task.traces_dir)
+        except Exception as e:
+            notification_manager.debug(
+                f"[TaskRunner] Failed to clean up trace files for task {task_id}: {e}"
+            )
+
         # Update internal tracking
         self._task_results[task_id] = result
+        self.task_results[task_id] = result
+
+        # Signal that a task has completed (for event-driven scheduling)
+        if self._task_completed_event:
+            self._task_completed_event.set()
 
         if result.status == TaskStatus.COMPLETED:
             self._completed_tasks.add(task_id)
+            self.completed_tasks.add(task_id)
             notification_manager.success(
                 f"[TaskRunner] Task completed successfully: {task_id} (duration: {result.duration:.2f}s)"
             )
         elif result.status == TaskStatus.TIMEOUT:
             self._failed_tasks.add(task_id)
+            self.failed_tasks.add(task_id)
             notification_manager.warning(
                 f"[TaskRunner] Task timed out: {task_id} (duration: {result.duration:.2f}s)"
             )
         elif result.status == TaskStatus.MEMORY_LIMIT_EXCEEDED:
             self._failed_tasks.add(task_id)
+            self.failed_tasks.add(task_id)
             notification_manager.warning(
                 f"[TaskRunner] Task exceeded memory limit: {task_id} (duration: {result.duration:.2f}s)"
             )
+        elif result.status == TaskStatus.SIGNAL_INTERRUPTED:
+            self._failed_tasks.add(task_id)
+            self.failed_tasks.add(task_id)
+            notification_manager.warning(
+                f"[TaskRunner] Task interrupted by signal: {task_id} (duration: {result.duration:.2f}s)"
+            )
         else:
             self._failed_tasks.add(task_id)
+            self.failed_tasks.add(task_id)
             notification_manager.error(
                 f"[TaskRunner] Task failed: {task_id} (status: {result.status.value}, "
                 f"return_code: {result.return_code}, duration: {result.duration:.2f}s)"
@@ -420,32 +501,51 @@ class TaskRunner:
 
     async def _force_kill_all_tasks(self) -> None:
         """
-        Force kill all running tasks immediately.
+        Force kill all running tasks with brief grace period.
 
-        Cancels all asyncio tasks and kills underlying processes via ProcessManager.
+        Gives tasks a 2-second grace period to complete naturally before
+        canceling asyncio tasks and killing underlying processes.
         """
         if not self._running_tasks:
             return
 
-        # Cancel all asyncio tasks immediately
         running_tasks: List[asyncio.Task[TaskResult]] = list(
             self._running_tasks.values()
         )
 
-        for task in running_tasks:
-            if not task.done():
-                task.cancel()
+        # Give tasks a brief grace period to complete naturally
+        notification_manager.info(
+            f"[TaskRunner] Giving {len(running_tasks)} running tasks 2 seconds to complete before force termination..."
+        )
 
-        # Force kill all processes via ProcessManager
-        await process_manager.kill_all_processes()
-
-        # Wait briefly for cancellations to complete
         if running_tasks:
-            _, pending = await asyncio.wait(running_tasks, timeout=5.0)
+            done, pending = await asyncio.wait(running_tasks, timeout=2.0)
+
+            # Process any tasks that completed during grace period
+            for task in done:
+                if not task.cancelled():
+                    try:
+                        # Task completed naturally - let normal completion handling work
+                        await task
+                    except Exception:
+                        # Task failed during grace period, still better than cancellation
+                        pass
+
+            # Cancel remaining tasks that didn't complete
+            for task in pending:
+                if not task.done():
+                    task.cancel()
+
+            # Force kill all remaining processes via ProcessManager
+            await process_manager.kill_all_processes()
+
+            # Wait briefly for cancellations to complete
             if pending:
-                notification_manager.warning(
-                    "[TaskRunner] Some tasks did not respond to cancellation within timeout"
-                )
+                _, still_pending = await asyncio.wait(pending, timeout=3.0)
+                if still_pending:
+                    notification_manager.warning(
+                        f"[TaskRunner] {len(still_pending)} tasks did not respond to cancellation within timeout"
+                    )
 
         # Clear all tracking - both running and pending tasks
         self._running_tasks.clear()
